"""
    `minion-ci` is a minimalist, decentralized, flexible Continuous Integration Server for hackers.

    :copyright: (c) by Timo Furrer
    :license: MIT, see LICENSE for details
"""

__VERSION__ = "0.0.2"
