"""
    `minion-ci` is a minimalist, decentralized, flexible Continuous Integration Server for hackers.

    This package contains the minion-ci server.

    :copyright: (c) by Timo Furrer
    :license: MIT, see LICENSE for details
"""
